@@ -35,10 +35,12 @@ use Boot::Lilo;
 use Boot::Grub;
 use Boot::Palo;
 use Boot::Aboot;
+use Boot::ZIPL;
+use Boot::Kboot;
 
 use vars qw(@boottypes $VERSION);
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.20 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d", q$Revision: 697 $ =~ /(\d+)/);
 
 sub install_config {
     my $config = shift;
