@@ -1,6 +1,6 @@
 package Boot::YaBoot;
 
-#   $Header: /cvsroot/systemconfig/systemconfig/lib/Boot/YaBoot.pm,v 1.8 2002/10/29 19:56:59 sdague Exp $
+#   $Header$
 
 #   Copyright (c) 2001 International Business Machines
 
@@ -47,7 +47,7 @@ use Boot;
 use Util::Cmd qw(:all);
 use Util::Log qw(:print);
 
-$VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d", q$Revision: 668 $ =~ /(\d+)/);
 
 push @Boot::boottypes, qw(Boot::YaBoot);
 
@@ -83,6 +83,8 @@ sub new {
                 $this{bootloader_exe} = "$this{root}/boot/yaboot.$this{platform_t}";
             } elsif (-e $this{root}."/boot/yaboot") {
                 $this{bootloader_exe} = "$this{root}/boot/yaboot";
+            } elsif (-e $this{root}."/usr/lib/yaboot/yaboot") {
+                $this{bootloader_exe} = "$this{root}/usr/lib/yaboot/yaboot";
             } else {
                 verbose("Read README file for naming of Yaboot image.");
                 carp("Cannot find a Yaboot image.\n");
